/*******************************************************************************
 * Copyright 2023 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef VOXL_TOF_SERVER_PIPE_INTERFACE_H
#define VOXL_TOF_SERVER_PIPE_INTERFACE_H


#include <modal_pipe_interfaces.h>
#include <modal_pipe_common.h> // for MODAL_PIPE_DEFAULT_BASE_DIR
#include <stdio.h>
#include <rc_math/quaternion.h>


#define TOF_PIPE_NAME		"rangefinder_pc"
// using the pipe tof_pc2 during development to see the visualizations
// on voxl-portal
//#define TOF_PIPE_NAME		"tof_pc2"
#define TOF_PIPE_LOCATION	(MODAL_PIPE_DEFAULT_BASE_DIR TOF_PIPE_NAME "/")

/**
 * Unique 32-bit number used to signal the beginning of a data packet while
 * parsing a data stream. If this were to be cast as a float it would have a
 * value of 5.7x10^13 which is an impossible value for translations/rotation
 * readings making it unique as an identifier.
 *
 * Also it spells "VOXL" in ASCII
 */
#define TOF_MAGIC_NUMBER (0x564F584C)


#define TOF_TYPE_STRINGS {"unknown", "TOF_VL53L1X", "TOF_VL53L7CX", "TOF_VL53L8CX"}
#define N_TOF_TYPES	4

// types for the 'type' fields in tof_measurement_t
#define TOF_TYPE_UNKNOWN		0
#define TOF_TYPE_VL53L1X		1
#define TOF_TYPE_VL53L7CX		2
#define TOF_TYPE_VL53L8CX		3

/*
 * @brief   This struct is created from the received tof_measurement_t
 *          packets and holds pointers to the data waiting in the pipe
 *          and the vectors necessary to compute the relative positions
 *          of the measurements.
 */
typedef struct {
    rc_vector_t location;
    rc_vector_t tb_angle;
    double angle;
    __int16_t * distance_mm;
    __int16_t * uncertainty_mm;
    int n_measurements;
    int n_outres;
    float fov_x;
    float fov_y;
    rc_vector_t direction_q;
} pt_context;

// we recommend a 128k buffer that can hold 1927 rangefinder packets
#define TOF_SERVER_RECOMMENDED_PIPE_SIZE		(128 * 1024)


#endif // VOXL_TOF_SERVER_PIPE_INTERFACE_H
